"""
This file will be copied to a temporary directory in order to
exercise caching compiled C callbacks.

See test_cfunc.py.
"""
from __future__ import division, print_function, absolute_import

import sys

from numba import cfunc, jit
from numba.tests.support import TestCase, captured_stderr


Z = 1

add_sig = "float64(float64, float64)"

div_sig = "float64(int64, int64)"


@cfunc(add_sig, cache=True, nopython=True)
def add_usecase(x, y):
    return x + y + Z

@cfunc(add_sig, nopython=True)
def add_nocache_usecase(x, y):
    return x + y + Z

@cfunc(div_sig, cache=True, nopython=True)
def div_usecase(a, b):
    return a / b


@jit(nopython=True)
def inner(x, y):
    return x + y + Z

@cfunc(add_sig, cache=True, nopython=True)
def outer(x, y):
    return inner(-y, x)


class _TestModule(TestCase):
    """
    Tests for functionality of this module's cfuncs.
    Note this does not define any "test_*" method, instead check_module()
    should be called by hand.
    """

    def check_module(self, mod):
        f = mod.add_usecase
        self.assertPreciseEqual(f.ctypes(2.0, 3.0), 6.0)
        f = mod.add_nocache_usecase
        self.assertPreciseEqual(f.ctypes(2.0, 3.0), 6.0)
        f = mod.outer
        self.assertPreciseEqual(f.ctypes(5.0, 2.0), 4.0)

        f = mod.div_usecase
        with captured_stderr() as err:
            self.assertPreciseEqual(f.ctypes(7, 2), 3.5)
        self.assertEqual(err.getvalue(), "")
        with captured_stderr() as err:
            f.ctypes(7, 0)
        err = err.getvalue()
        self.assertIn("ZeroDivisionError", err)

    # For 2.x
    def runTest(self):
        raise NotImplementedError


def self_test():
    mod = sys.modules[__name__]
    _TestModule().check_module(mod)



from __future__ import print_function

import random

import numpy as np

from .support import TestCase
from numba import njit
from numba.unsafe.tuple import tuple_setitem
from numba.unsafe.ndarray import to_fixed_tuple
from numba.errors import TypingError


class TestTupleIntrinsic(TestCase):
    """Tests for numba.unsafe.tuple
    """
    def test_tuple_setitem(self):
        @njit
        def foo(tup, idxs, vals):
            out_tup = tup
            for i, v in zip(idxs, vals):
                out_tup = tuple_setitem(out_tup, i, v)
            return tup, out_tup

        random.seed(123)
        for _ in range(20):
            # Random data
            n = random.randint(1, 10)
            tup = tuple([random.randint(0, n) for i in range(n)])
            vals = tuple([random.randint(10, 20) for i in range(n)])
            idxs = list(range(len(vals)))
            random.shuffle(idxs)
            idxs = tuple(idxs)
            # Expect
            expect_tup = tuple(tup)
            expect_out = np.asarray(expect_tup)
            expect_out[np.asarray(idxs)] = vals
            # Got
            got_tup, got_out = foo(tup, idxs, vals)
            # Check
            self.assertEqual(got_tup, expect_tup)
            self.assertEqual(got_out, tuple(expect_out))


class TestNdarrayIntrinsic(TestCase):
    """Tests for numba.unsafe.ndarray
    """
    def test_to_fixed_tuple(self):
        const = 3

        @njit
        def foo(array):
            a = to_fixed_tuple(array, length=1)
            b = to_fixed_tuple(array, 2)
            c = to_fixed_tuple(array, const)
            d = to_fixed_tuple(array, 0)
            return a, b, c, d

        np.random.seed(123)
        for _ in range(10):
            # Random data
            arr = np.random.random(3)
            # Run
            a, b, c, d = foo(arr)
            # Check
            self.assertEqual(a, tuple(arr[:1]))
            self.assertEqual(b, tuple(arr[:2]))
            self.assertEqual(c, tuple(arr[:3]))
            self.assertEqual(d, ())

        # Check error with ndim!=1
        with self.assertRaises(TypingError) as raises:
            foo(np.random.random((1, 2)))
        self.assertIn("Not supported on array.ndim=2",
                      str(raises.exception))

        # Check error with non-constant length
        @njit
        def tuple_with_length(array, length):
            return to_fixed_tuple(array, length)

        with self.assertRaises(TypingError) as raises:
            tuple_with_length(np.random.random(3), 1)
        expectmsg = "RequireConstValue: *length* argument must be a constant"
        self.assertIn(expectmsg, str(raises.exception))
